<?php 

class pisol_fsnw_shippingzone{

    public $selected_shipping_zone;
    public $free_shipping_class;
    public $free_shipping_minimum_order;
    public $is_min_amount_and_coupon = false;
    public $ignore_discounts;
    public $is_min_amount_or_coupon;
    

    function __construct(){
        /* we get the selected shipping zone */
        $this->selected_shipping_zone = $this->getUserSelectedClass();

        /* if user has not yet selected a shipping zone then we use default shipping zone from setting */
        $this->selected_shipping_zone = $this->selected_shipping_zone == false ? $this->getDefaultShippingZone() : $this->selected_shipping_zone;
        $this->free_shipping_class = $this->getFreeShippingClass();
        $this->free_shipping_minimum_order = $this->getFreeShippingMinimum();
        $this->is_min_amount_and_coupon = $this->minAmountAndCoupon();
        $this->is_min_amount_or_coupon = $this->minAmountORCoupon();
        $this->ignore_discounts = $this->ignoreDiscounts();
    }

    function ignoreDiscounts(){
        if ( $this->free_shipping_class ) {
            foreach($this->free_shipping_class as $shipping_class){
                $first_zone       = $shipping_class;
                $instance_id      = $first_zone->instance_id;
                $method_id        = $first_zone->method_id;
                $arr_method       = array( $method_id, $instance_id );
                $implode_method   = implode( "_", $arr_method );
                $free_option      = 'woocommerce_' . $implode_method . '_settings';
                $free_shipping_s  = get_option( $free_option );
                if(isset($free_shipping_s['ignore_discounts']) && $free_shipping_s['ignore_discounts'] == 'yes') return 'yes';
            }
        }
        return 'no';
    }

    function checkDefaultShippingZone(){
        $default  = get_option('pi_fsnw_default_shipping_zone', false);
        if($default == "" || $default == false ){
            return false;
        }
        return true;
    }

     function getDefaultShippingZone(){
        if($this->checkDefaultShippingZone()) return get_option('pi_fsnw_default_shipping_zone',false);
        return false;        
    }

    function getUserSelectedClass(){
        global $woocommerce;
        
       
        $shipping_zone_id = false;

		if( isset(WC()->session)):

		if((!empty(WC()->session) && is_array(WC()->session->get( 'chosen_shipping_methods' ))) || (isset($_GET['wc-ajax']) && $_GET['wc-ajax'] == 'update_order_review') ) {

            if($shipping_zone_id === false){
                $shipping_zone_id = $this->getUserSelectedZone();
            }
    
            if($shipping_zone_id === false){
                $shipping_zone_id = $this->getGeoLocatedZone();
            }
		}elseif(!empty(WC()->session)){
			if($shipping_zone_id === false){
            	$shipping_zone_id = $this->getGeoLocatedZone();
        	}
		}
        endif;

        return $shipping_zone_id ;
		
	}

    private function getUserSelectedZone(){
        global $woocommerce;
        if(isset(WC()->cart) && is_object(WC()->cart)){
            $shipping_packages =  WC()->cart->get_shipping_packages();
        
            $shipping_zone = wc_get_shipping_zone( reset( $shipping_packages ) );

            if(is_object($shipping_zone)){
                return $shipping_zone->get_id();
            }
        }
        return false;
    }

    private function getGeoLocatedZone(){
        $destination = $this->ipBasedDestination();
        
        if(empty($destination['destination']['country'])) return false;

        $shipping_zone = WC_Shipping_Zones::get_zone_matching_package( $destination );
        if(is_object($shipping_zone)){
            return $shipping_zone->get_id();
        }

        return false;
    }

    private function ipBasedDestination(){
        $geo_instance  = new WC_Geolocation();
        $user_ip  = $geo_instance->get_ip_address();
        $user_geodata = $geo_instance->geolocate_ip($user_ip);
        
        $destination['destination']['country'] =  $user_geodata['country'];
        $destination['destination']['state'] =  $user_geodata['state'];
        $destination['destination']['postcode'] = "";
        return $destination;
    }


     function checkFreeShippingAvailable(){
			global $wpdb;
			$wfspb_query = $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}woocommerce_shipping_zone_methods WHERE method_id = %s AND is_enabled = %d AND zone_id = %d ORDER BY method_order ASC", 'free_shipping', 1, $this->selected_shipping_zone );
			$zone_data   = $wpdb->get_results( $wfspb_query, OBJECT );

			if ( empty( $zone_data ) ) {
				return false;
			} else {
				return $zone_data;
			}

    }

     function getFreeShippingClass(){
        return $this->checkFreeShippingAvailable();
    }
    
    /**
     * This can return 2 thing
     * number > 0 = there is minimum restriction
     * 0 = this means free shipping is available to all
     * false = there is no free shipping method
     */
     function getFreeShippingMinimum(){
        if ( $this->free_shipping_class ) {
            foreach($this->free_shipping_class as $shipping_class){
                $first_zone       = $shipping_class;
                $instance_id      = $first_zone->instance_id;
                $method_id        = $first_zone->method_id;
                $arr_method       = array( $method_id, $instance_id );
                $implode_method   = implode( "_", $arr_method );
                $free_option      = 'woocommerce_' . $implode_method . '_settings';
                $free_shipping_s  = get_option( $free_option );
                if(isset($free_shipping_s['min_amount']) && $free_shipping_s['min_amount'] > 0 && isset($free_shipping_s['requires']) && ($free_shipping_s['requires'] == "min_amount" || $free_shipping_s['requires'] == "either"  || $free_shipping_s['requires'] == "both" )){
                    $order_min_amount = $free_shipping_s['min_amount'];
                    return (float)$order_min_amount;
                }elseif(!isset($free_shipping_s['requires']) || $free_shipping_s['requires'] == ""){
                    $order_min_amount = 0;
                    return $order_min_amount;
                }
            }

            return false;
        } else {
            return false;
        }
    }

    function minAmountAndCoupon(){
        if ( $this->free_shipping_class ) {
            foreach($this->free_shipping_class as $shipping_class){
                $first_zone       = $shipping_class;
                $instance_id      = $first_zone->instance_id;
                $method_id        = $first_zone->method_id;
                $arr_method       = array( $method_id, $instance_id );
                $implode_method   = implode( "_", $arr_method );
                $free_option      = 'woocommerce_' . $implode_method . '_settings';
                $free_shipping_s  = get_option( $free_option );
                if(isset($free_shipping_s['min_amount']) && $free_shipping_s['min_amount'] > 0 && isset($free_shipping_s['requires']) && $free_shipping_s['requires'] == "both"){
                    return true;
                }else{
                    return false;
                }
            }
        }
        return false;
    }

    function minAmountOrCoupon(){
        if ( $this->free_shipping_class ) {
            foreach($this->free_shipping_class as $shipping_class){
                $first_zone       = $shipping_class;
                $instance_id      = $first_zone->instance_id;
                $method_id        = $first_zone->method_id;
                $arr_method       = array( $method_id, $instance_id );
                $implode_method   = implode( "_", $arr_method );
                $free_option      = 'woocommerce_' . $implode_method . '_settings';
                $free_shipping_s  = get_option( $free_option );
                if(isset($free_shipping_s['min_amount']) && $free_shipping_s['min_amount'] > 0 && isset($free_shipping_s['requires']) && $free_shipping_s['requires'] == "either"){
                    return true;
                }else{
                    return false;
                }
            }
        }
        return false;
    }

}