<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              piwebsolution.com
 * @since             2.2.0
 * @package           Pisol_Fsnw
 *
 * @wordpress-plugin
 * Plugin Name:       Free shipping notification WooCommerce
 * Plugin URI:        piwebsolution.com/free-shipping-notification-documentation
 * Description:       Free shipping notification for WooCommerce will show a notification bar on the top, how much more you need to buy to get free shipping
 * Version:           2.2.0
 * Author:            PI Websolution
 * Author URI:        piwebsolution.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pisol-fsnw
 * Domain Path:       /languages
 * WC tested up to: 8.6.0
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/* 
    Making sure woocommerce is there 
*/
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

if(!is_plugin_active( 'woocommerce/woocommerce.php')){
    function pi_fsnw_free_woocommerce_error_notice() {
        ?>
        <div class="error notice">
            <p><?php _e( 'Please Install and Activate WooCommerce plugin, without that Free shipping notification WooCommerce plugin cant work', 'pisol-fsnw' ); ?></p>
        </div>
        <?php
    }
    add_action( 'admin_notices', 'pi_fsnw_free_woocommerce_error_notice' );
    return;
}else{

if(is_plugin_active( 'free-shipping-notification-woocommerce-pro/pisol-fsnw.php')){
    function pi_pisol_pro_error_notice() {
        ?>
        <div class="error notice">
            <p><?php _e( 'You have the PRO version of this plugin in your site', 'pisol-fsnw'); ?></p>
        </div>
        <?php
    }
    add_action( 'admin_notices', 'pi_pisol_pro_error_notice' );
    deactivate_plugins(plugin_basename(__FILE__));
    return;
}else{

/**
 * Declare compatible with HPOS new order table 
 */
add_action( 'before_woocommerce_init', function() {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );    

/**
 * Currently plugin version.
 * Start at version 2.2.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PISOL_FSNW_VERSION', '2.2.0' );
define( 'PISOL_FSNW_DELETE_SETTING', false );
define('PI_FSNW_BUY_URL', 'https://www.piwebsolution.com/product/free-shipping-notification-bar/');
define('PI_FSNW_PRICE', '$19');
/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pisol-fsnw-activator.php
 */
function activate_pisol_fsnw() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pisol-fsnw-activator.php';
	Pisol_Fsnw_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-pisol-fsnw-deactivator.php
 */
function deactivate_pisol_fsnw() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pisol-fsnw-deactivator.php';
	Pisol_Fsnw_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_pisol_fsnw' );
register_deactivation_hook( __FILE__, 'deactivate_pisol_fsnw' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-pisol-fsnw.php';

function pisol_fsnw_plugin_link( $links ) {
	$links = array_merge( array(
        '<a href="' . esc_url( admin_url( '/admin.php?page=pisol-fsnw-notification' ) ) . '">' . __( 'Settings', 'pisol-fsnw' ) . '</a>',
        '<a style="color:#0a9a3e; font-weight:bold;" target="_blank" href="' . esc_url(PI_FSNW_BUY_URL) . '">' . __( 'Buy PRO Version', 'pisol-fsnw' ) . '</a>',
        '<a style="color:#0a9a3e; font-weight:bold;" target="_blank" href="https://wordpress.org/support/plugin/free-shipping-notification-woocommerce/reviews/#bbp_topic_content">' . __( 'Give Suggestions','pisol-fsnw' ) . '</a>'
	), $links );
	return $links;
}
add_action( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'pisol_fsnw_plugin_link' );




/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_pisol_fsnw() {

	$plugin = new Pisol_Fsnw();
	$plugin->run();

}
run_pisol_fsnw();

}
}