<?php

class pisol_fsnw_cart_checkout_progress_bar{

    public $enable_cart;
    public $enable_checkout;
    public $cart_position;
    public $checkout_position;
    public $min_order;
    public $total;
    public $missing_amount;
    public $is_min_amount_and_coupon;
    public $is_min_amount_or_coupon;
    public $free_shipping_coupon_added;
    
    function __construct(){
        $this->enable_cart = get_option('pi_fsnw_inside_cart_total', 0);
        $this->enable_checkout = get_option('pi_fsnw_inside_checkout_total', 0);
        $this->cart_position = get_option('pi_fsnw_inside_cart_position','woocommerce_cart_totals_before_shipping');
        $this->checkout_position = get_option('pi_fsnw_inside_checkout_position','woocommerce_review_order_before_shipping');
        if(!empty($this->enable_cart)){
            add_action($this->cart_position, array($this, 'cartNotificationBar'));
        }

        if(!empty($this->enable_checkout)){
            add_action($this->checkout_position, array($this, 'checkoutNotificationBar'));

            /**
             * needed as when shipping method changes shipping method based discount rule get applied based on the old shipping method
             */
            add_filter('woocommerce_update_order_review_fragments', array(__CLASS__, 'detectChangeShippingMethod'));
        }

        
    }

    function template($message, $type = ''){

        $progress_title = get_option('pi_fsnw_cc_progress_bar_text', 'Free shipping');

        if($this->min_order === false) return;

        if($this->min_order !== false){
            if($this->min_order > 0 ){
                $percent = ($this->total / $this->min_order)*100;
            }else{
                $percent = 100;
            }
        }else{
            $percent = false;
        }

        if($percent > 100) $percent = 100;

        if(is_numeric($percent)){
            $progress_title = str_replace('{percent}', number_format($percent,2), $progress_title);
        }


        if(empty($type)){
        printf('<div class="pi-fsnw-cart-message">%s</div>', $message);
        printf('<div class="pi-fsnw-container"><div class="pi-inner-content">%s</div>
        <div class="pi-fsnw-container-progress" style="height:24px;width:%s%%"></div></div>', $progress_title, $percent);
        }else{
            echo '<tr id="pi-fsnw-row"><td colspan="2">';
            printf('<div class="pi-fsnw-cart-message">%s</div>', $message);
            printf('<div class="pi-fsnw-container"><div class="pi-inner-content">%s</div>
            <div class="pi-fsnw-container-progress" style="height:24px;width:%s%%"></div></div>', $progress_title, $percent);
            echo '</td></tr>';
        }
    }

    function cartNotificationBar(){
        $this->notificationBar('table');
    }

    function checkoutNotificationBar(){
        if(!is_ajax()) return;
        $this->notificationBar('table');
    }

    function notificationBar($type = ''){
            $this->initializeFunction();
            $final_message = $this->createMessage();

            $this->template($final_message, $type);
    }

    function initializeFunction(){
        $shipping_obj = new pisol_fsnw_shippingzone();
        $this->min_order = $shipping_obj->free_shipping_minimum_order;
        $this->total = $this->getCartTotal($shipping_obj);
        $this->missing_amount = $this->missingAmount();
        $this->is_min_amount_and_coupon = $shipping_obj->is_min_amount_and_coupon;
        $this->is_min_amount_or_coupon = $shipping_obj->is_min_amount_or_coupon;
        $this->free_shipping_coupon_added = pisl_fsnw_message::isFreeShippingCouponAdded();

        if($this->is_min_amount_or_coupon && $this->free_shipping_coupon_added){
            $this->missing_amount = 0;
            $this->min_order = $this->total;
        }
    }

    function createMessage(){
           
    
        $values = array(
            'minimum_order' => wc_price($this->min_order),
            'cart_total' => wc_price($this->total),
            'missing_amount' => wc_price($this->missing_amount)
        );

        $message = $this->getMessageText();

        $final_message = self::searchReplace($message, $values);
        return wp_kses_post($final_message);
    }

    static function searchReplace($message, $values){

        foreach($values as $key => $value){
            $open_tag = '<span class="pisol_shortcodes pisol_'.$key.'">';
            $close_tag = '</span>';
            $message = str_replace('{'.$key.'}', $open_tag.$value.$close_tag, $message);
        }
        return $message;
    }

    function getMessageText(){
        $cart_page_id = wc_get_page_id( 'checkout' );
        $cart_page_url = $cart_page_id ? get_permalink( $cart_page_id ) : '';

        if($this->min_order === false){
            return "";
        }

        if($this->min_order > 0){
            $percent  = ( $this->total / $this->min_order ) *100;
        }else{
            $percent = 100;
        }

        if($percent == 0 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_exact_0_and', 'Free shipping for order above {minimum_order} with a coupon code');
            }else{
                $message = get_option('pi_fsnw_message_exact_0', 'Order above {minimum_order} gets free shipping. Add {missing_amount} for free shipping.');
            }
        }elseif($percent > 0 && $percent <= 50 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_exact_0_and', 'Free shipping for order above {minimum_order} with a coupon code');
            }else{
                $message = get_option('pi_fsnw_message_exact_0', 'Order above {minimum_order} gets free shipping. Add {missing_amount} for free shipping.');
            }
        }elseif($percent >  50 && $percent < 100 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_exact_0_and', 'Free shipping for order above {minimum_order} with a coupon code');
            }else{
                $message = get_option('pi_fsnw_message_exact_0', 'Order above {minimum_order} gets free shipping. Add {missing_amount} for free shipping.');
            }
        }elseif($percent >= 100 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_100_and', 'You are now qualified for the Free shipping, go to <a href="'.$cart_page_url.'">Checkout</a> and add the free shipping coupon code');
            }else{
                $message = get_option('pi_fsnw_message_100', 'You are now qualified for the Free shipping, go to <a href="'.$cart_page_url.'">Checkout</a>');
            }
        }elseif($percent >= 100 && $this->min_order === 0){
            $message = get_option('pi_fsnw_normal_free_shipping_message', 'Free Shipping');
        }

        return $message;
    }

    /**
    * We cant take woocommerce total as it has shipping included in it, 
    * so we do our own calculation of total without shipping in it
    */
    function getCartTotal($shipping_obj){
        
        if(!isset(WC()->cart)) return 0;

        $total = WC()->cart->get_displayed_subtotal();
        
		if ( WC()->cart->display_prices_including_tax() ) {
            $total = $total - WC()->cart->get_discount_tax();
        }

        if ( 'no' === $shipping_obj->ignore_discounts ) {
            $total = $total - WC()->cart->get_discount_total();
        }

        $total = pisl_fsnw_message::round( $total, wc_get_price_decimals() );

        return $total;
    }

    function missingAmount(){
        if($this->min_order === false) return false;
        
        if($this->min_order > 0){
            $missing_amount = $this->min_order - $this->total;
        }else{
            $missing_amount = 0;
        }
        return $missing_amount;
    }

    static function detectChangeShippingMethod($values){
        $chosen_method = WC()->session->get( 'chosen_shipping_methods' );
        $values['old_method'] = self::oldShippingMethod();
        $values['new_method'] = (is_array($chosen_method) && !empty($chosen_method)) ? $chosen_method[0] : false;
        $shipping_methods = self::getShippingMethods();
        $values['count'] = count($shipping_methods);
        return $values;
    }

    static function oldShippingMethod(){
        $shipping_method = false;
        if(isset($_POST['post_data'])){
            parse_str($_POST['post_data'], $values);
            $shipping_method = isset($values['shipping_method']) && is_array($values['shipping_method']) ? $values['shipping_method'][0] : false;
        }
        return $shipping_method;
    }

    static function getShippingMethods(){
        if(function_exists('WC') && is_object(WC()->shipping)){
            $packages = WC()->shipping->get_packages();
            $shipping_methods = array();
            foreach($packages as $package){
                if(empty($package['rates']) || !is_array($package['rates'])) break;

                foreach($package['rates'] as $id => $rate){
                    $title = wc_cart_totals_shipping_method_label($rate);
                    $shipping_methods[$id] = $title;
                }
            }
            return $shipping_methods;
        }
        return array();
	}
}

new pisol_fsnw_cart_checkout_progress_bar();