<?php

class pisl_fsnw_message{

    public $min_order;
    public $total;
    public $missing_amount;
    public $is_min_amount_and_coupon;
    public $is_min_amount_or_coupon;
    public $free_shipping_coupon_added;
    

    function __construct(){
        $free_shipping_notification = get_option('pi_fsnw_enabled',1);
        $pi_fsnw_default_shipping_zone = get_option('pi_fsnw_default_shipping_zone',0);

        if(!empty($free_shipping_notification) /*&& $pi_fsnw_default_shipping_zone != 0 */){
            add_action( 'wp_ajax_nopriv_get_cart_fsnw', array( $this, 'ajaxResponse' ) );
            add_action( 'wp_ajax_get_cart_fsnw', array( $this, 'ajaxResponse' ) );
            add_action( 'wc_ajax_get_cart_fsnw', array( $this, 'ajaxResponse' ) );
            add_action( 'wp_footer', array( $this, 'loadBarOnPage' ), 90 );
            add_action( 'wp_enqueue_scripts', array( $this,  'inlineCss') );
            add_action( 'wp_enqueue_scripts', array( $this,  'inlineJs') );
        }
    }

    function initializeFunction(){
        $shipping_obj = new pisol_fsnw_shippingzone();
        $this->min_order = $shipping_obj->free_shipping_minimum_order;
        $this->total = $this->getCartTotal($shipping_obj);
        $this->missing_amount = $this->missingAmount();
        $this->is_min_amount_and_coupon = $shipping_obj->is_min_amount_and_coupon;
        $this->is_min_amount_or_coupon = $shipping_obj->is_min_amount_or_coupon;
        $this->free_shipping_coupon_added = self::isFreeShippingCouponAdded();

        if($this->is_min_amount_or_coupon && $this->free_shipping_coupon_added){
            $this->missing_amount = 0;
            $this->min_order = $this->total;
        }
    }

    static function isFreeShippingCouponAdded(){
        if(function_exists('WC') && isset(WC()->cart)){
            $coupons = WC()->cart->get_coupons();

            if ( $coupons ) {
                foreach ( $coupons as $code => $coupon ) {
                    if ( $coupon->is_valid() && $coupon->get_free_shipping() ) {
                        return true;
                        break;
                    }
                }
            }
        }
        return false;
    }

    function ajaxResponse(){
        $final_message = $this->createMessage();
        if($this->min_order !== false){
            if($this->min_order > 0 ){
                $percent = ($this->total / $this->min_order)*100;
            }else{
                $percent = 100;
            }
        }else{
            $percent = false;
        }
        $json = array('message_bar'=> $final_message, 'min_order'=> $this->min_order, 'total'=> $this->total, 'percent'=>$percent);
        echo json_encode($json);
        die;
    }

    /**
     * This is must as it has initialize function 
     * that sets various variables
     */
    function createMessage(){
        $this->initializeFunction();
        $values = array(
            'minimum_order' => wc_price($this->min_order),
            'cart_total' => wc_price($this->total),
            'missing_amount' => wc_price($this->missing_amount)
        );
        $message = $this->getMessageText();

        $final_message = $this->searchReplace($message, $values);
        return wp_kses_post($final_message);
    }

    function getMessageText(){
        $cart_page_id = wc_get_page_id( 'checkout' );
        $cart_page_url = $cart_page_id ? get_permalink( $cart_page_id ) : '';

        if($this->min_order === false){
            return "";
        }

        if($this->min_order > 0){
            $percent  = ( $this->total / $this->min_order ) *100;
        }else{
            $percent = 100;
        }

        if($percent == 0 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_exact_0_and','Free shipping for order above {minimum_order} with a coupon code');
            }else{
                $message = get_option('pi_fsnw_message_exact_0','Order above {minimum_order} gets free shipping. Add {missing_amount} for free shipping.');
            }
        }elseif($percent > 0 && $percent <= 50 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_exact_0_and','Free shipping for order above {minimum_order} with a coupon code');
            }else{
                $message = get_option('pi_fsnw_message_exact_0','Order above {minimum_order} gets free shipping. Add {missing_amount} for free shipping.');
            }
        }elseif($percent >  50 && $percent < 100 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_exact_0_and','Free shipping for order above {minimum_order} with a coupon code');
            }else{
                $message = get_option('pi_fsnw_message_exact_0','Order above {minimum_order} gets free shipping. Add {missing_amount} for free shipping.');
            }
        }elseif($percent >= 100 && $this->min_order > 0){
            if($this->is_min_amount_and_coupon){
                $message = get_option('pi_fsnw_message_100_and','You are now qualified for the Free shipping, go to <a href="'.$cart_page_url.'">Checkout</a> and add the free shipping coupon code');
            }else{
                $message = get_option('pi_fsnw_message_100','You are now qualified for the Free shipping, go to <a href="'.$cart_page_url.'">Checkout</a>');
            }
        }elseif($percent >= 100 && $this->min_order === 0){
            $message = get_option('pi_fsnw_normal_free_shipping_message','Free Shipping');
        }

        return apply_filters('pisol_fsnw_message_filter', $message, $this);
    }

    function searchReplace($message, $values){

        foreach($values as $key => $value){
            $open_tag = '<span class="pisol_shortcodes pisol_'.$key.'">';
            $close_tag = '</span>';
            $message = str_replace('{'.$key.'}', $open_tag.$value.$close_tag, $message);
        }
        return $message;
    }

    /**
     * direct loading the message in the html on initial load
     * reduces load on server, we dont have to do ajax call untill user buy some thing or update cart
     */
    public function loadBarOnPage() {
        echo $this->messageTemplate("");
    }

    function messageTemplate($message){
        $show_close_button = get_option('pi_fsnw_close_button',1);

        $message_template = "<div class='pisol-bar-container'>
                <div class='pisol-bar-message'>{$message}</div>
                ".(!empty($show_close_button) ? "<a href='javascript:void(0);' class='pisol-fsnw-close'>&times;</a>" : "")."
              </div>
        ";
        $message_template .= "<div id='pi-progress-circle' style='background-image:url(".plugin_dir_url( __FILE__ )."img/free-delivery.png);'></div>";
        return $message_template;
    }
    
    /**
    * We cant take woocommerce total as it has shipping included in it, 
    * so we do our own calculation of total without shipping in it
    */
    function getCartTotal($shipping_obj){
        
        if(!isset(WC()->cart)) return 0;

        $total = WC()->cart->get_displayed_subtotal();
        
		if ( WC()->cart->display_prices_including_tax() ) {
            $total = $total - WC()->cart->get_discount_tax();
        }

        if ( 'no' === $shipping_obj->ignore_discounts ) {
            $total = $total - WC()->cart->get_discount_total();
        }

        $total = self::round( $total, wc_get_price_decimals() );

        return $total;
    }

    public static function round( $val, int $precision = 0, int $mode = PHP_ROUND_HALF_UP ) : float {
		if ( ! is_numeric( $val ) ) {
			$val = floatval( $val );
		}
		return round( $val, $precision, $mode );
	}

    function missingAmount(){
        if($this->min_order === false) return false;
        
        if($this->min_order > 0){
            $missing_amount = $this->min_order - $this->total;
        }else{
            $missing_amount = 0;
        }
        return $missing_amount;
    }

    function inlineCss(){
        $pi_fsnw_position = esc_html(get_option("pi_fsnw_position",'top'));

        $pi_fsnw_background_color = esc_html(get_option("pi_fsnw_background_color",'#ee6443'));

        $pi_fsnw_font_color = esc_html(get_option("pi_fsnw_font_color",'#ffffff'));
        $pi_fsnw_shortcode_color = esc_html(get_option("pi_fsnw_shortcode_color",'#000000'));
        $pi_fsnw_link_color = esc_html(get_option("pi_fsnw_link_color",'#ffffff'));
        $pi_fsnw_close_color = esc_html(get_option("pi_fsnw_close_color",'#ffffff'));

        $pi_fsnw_font_weight = esc_html(get_option("pi_fsnw_font_weight",'normal'));
        $pi_fsnw_shortcode_weight = esc_html(get_option("pi_fsnw_shortcode_weight",'bold'));
        $pi_fsnw_link_weight = esc_html(get_option("pi_fsnw_link_weight",'normal'));
        $pi_fsnw_close_weight = esc_html(get_option("pi_fsnw_close_weight",'bold'));

        $pi_fsnw_font_size = esc_html(get_option("pi_fsnw_font_size",'16'));
        $pi_fsnw_shortcode_size = esc_html(get_option("pi_fsnw_shortcode_size",'16'));
        $pi_fsnw_link_size = esc_html(get_option("pi_fsnw_link_size",'16'));
        $pi_fsnw_close_size = esc_html(get_option("pi_fsnw_close_size",'22'));

        $css = "
            .pisol-bar-container{
                {$pi_fsnw_position}: 0px !important;
                background-color:{$pi_fsnw_background_color};
                color:{$pi_fsnw_font_color};
                font-weight:{$pi_fsnw_font_weight};
                font-size:{$pi_fsnw_font_size}px;
            }

            .pisol-bar-container a{
                color:{$pi_fsnw_link_color};
                font-weight:{$pi_fsnw_link_weight};
                font-size:{$pi_fsnw_link_size}px;
            }

            .pisol_shortcodes{
                color:{$pi_fsnw_shortcode_color};
                font-weight:{$pi_fsnw_shortcode_weight};
                font-size:{$pi_fsnw_shortcode_size}px;
            }

            .pisol-bar-container a.pisol-fsnw-close{
                color:{$pi_fsnw_close_color};
                font-weight:{$pi_fsnw_close_weight};
                font-size:{$pi_fsnw_close_size}px;
            }
        ";
        wp_register_style( 'pi-fsnw-dummy', false );
        wp_enqueue_style( 'pi-fsnw-dummy' );
        wp_add_inline_style('pi-fsnw-dummy' , $css );
    }

    function inlineJs(){
        $this->initializeFunction();
        if($this->min_order !== false){
            if($this->min_order > 0){
                $percent = ($this->total / $this->min_order)*100;
            }else{
                $percent = 100;
            }
            $values = array(
                'wc_ajax_url' => WC_AJAX::get_endpoint( '%%endpoint%%' ),
                'ajax_url'=>admin_url('admin-ajax.php'),
                'showContinues' =>(bool)(empty(get_option('pi_fsnw_persistent_bar',0)) ? false : true),
                'howLongToShow' =>(int)(((int)get_option('pi_fsnw_how_long_to_show',6))*1000),
                'percent'=> $percent
            );
        }else{
            $values = array(
                'wc_ajax_url' => WC_AJAX::get_endpoint( '%%endpoint%%' ),
                'ajax_url'=>admin_url('admin-ajax.php'),
                'showContinues' =>(bool)(empty(get_option('pi_fsnw_persistent_bar',0)) ? false : true),
                'howLongToShow' =>(int)(((int)get_option('pi_fsnw_how_long_to_show',6))*1000),
                'percent'=> 0
            );
        }

        wp_enqueue_script( 'pisol-fsnw-circle', plugin_dir_url( __FILE__ ) . 'js/circle-progress.min.js', array( 'jquery' ), PISOL_FSNW_VERSION, false );
        wp_enqueue_script( 'pisol-fsnw', plugin_dir_url( __FILE__ ) . 'js/pisol-fsnw-public.js', array('pisol-fsnw-circle', 'jquery' ), PISOL_FSNW_VERSION, false );

        wp_localize_script('pisol-fsnw', 'pisol_fsnw', $values);
    }

}